@@ -29,10 +29,19 @@ If t will update since last modified time, and if nil will update since last ent
   :group 'elfeed-protocol
   :type 'boolean)
 
+(defcustom elfeed-protocol-owncloud-fetch-category-as-tag t
+  "If true, tag the ownCloud News feed category to feed item."
+  :group 'elfeed-protocol
+  :type 'boolean)
+
+(defvar elfeed-protocol-owncloud-categories (make-hash-table :test 'equal)
+  "Category list from ownCloud News, will be used to tag entries with their ownCloud News category.")
+
 (defvar elfeed-protocol-owncloud-feeds (make-hash-table :test 'equal)
   "Feed list from ownCloud News, will be filled before updating operation.")
 
 (defconst elfeed-protocol-owncloud-api-base "/index.php/apps/news/api/v1-2")
+(defconst elfeed-protocol-owncloud-api-folders (concat elfeed-protocol-owncloud-api-base "/folders"))
 (defconst elfeed-protocol-owncloud-api-feeds (concat elfeed-protocol-owncloud-api-base "/feeds"))
 (defconst elfeed-protocol-owncloud-api-init-unread (concat elfeed-protocol-owncloud-api-base "/items?type=3&getRead=false&batchSize=%s"))
 (defconst elfeed-protocol-owncloud-api-init-starred (concat elfeed-protocol-owncloud-api-base "/items?type=2&getRead=true&batchSize=-1"))
@@ -91,7 +100,7 @@ finished."
                       (set-buffer-multibyte t))
                     (when elfeed-protocol-log-trace
                       (elfeed-log 'debug "elfeed-protocol-owncloud: %s" (buffer-string)))
-                    ,@body
+                    (elfeed-protocol-owncloud--parse-result ,@body)
                     (unless use-curl
                       (kill-buffer)))))))
      (if use-curl
@@ -107,14 +116,65 @@ finished."
          (let ((url-request-extra-headers headers))
            (url-retrieve no-auth-url cb () t t))))))
 
-(defun elfeed-protocol-owncloud--parse-feeds (host-url)
-  "Parse the feeds JSON buffer and fill results to db.
-HOST-URL is the host name of ownCloud server.  Ensure the point in the
-right place that `json-read' could execute.  Return
-`elfeed-protocol-owncloud-feeds'."
+(defmacro elfeed-protocol-owncloud--parse-result (&rest body)
+  "Parse owncloud api result JSON buffer.
+Will eval rest BODY expressions at end."
+  (declare (indent defun))
+  `(let* ((result (json-read))
+          (message (map-elt result 'message)))
+     (if (and message (or
+                       (equal message "")
+                       (equal message "CORS requires basic auth")))
+         (elfeed-log 'error "elfeed-protocol-owncloud: authentication failed, wrong username or password")
+       ,@body)))
+
+(defmacro elfeed-protocol-owncloud-fetch-prepare (host-url &rest body)
+  "Ensure logged in and feed list updated before expressions.
+HOST-URL is the host name of ownCloud server.  And will eval rest
+BODY expressions at end."
+  (declare (indent defun))
+  `(elfeed-protocol-owncloud--update-feed-list
+    ,host-url (lambda () ,@body)))
+
+(defun elfeed-protocol-owncloud--update-categories-list (host-url &optional callback)
+  "Update ownCloud News server categories list.
+HOST-URL is the host name of ownCloud server.  Will call CALLBACK at end."
+  (elfeed-log 'debug "elfeed-protocol-owncloud: update cagetory list")
+  (elfeed-protocol-owncloud-with-fetch
+    (concat host-url elfeed-protocol-owncloud-api-folders)
+    nil (elfeed-protocol-owncloud--parse-categories host-url result)
+    (when callback (funcall callback))))
+
+(defun elfeed-protocol-owncloud--parse-categories (host-url content)
+  "Parse the feeds JSON buffer and cache the result.
+HOST-URL is the host name of ownCloud server.  CONTENT is the result JSON content
+by http request.  Return cached `elfeed-protocol-owncloud-categories'."
   (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
-         (parsed-feeds (json-read))
-         (feeds (map-elt parsed-feeds 'feeds)))
+         (categories (map-elt content 'folders)))
+    (puthash proto-id categories elfeed-protocol-owncloud-categories)
+    elfeed-protocol-owncloud-categories))
+
+(defun elfeed-protocol-owncloud--get-category-name (host-url category-id)
+  "Return category name from HOST-URL for CATEGORY-ID."
+  (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
+         (categories (gethash proto-id elfeed-protocol-owncloud-categories))
+         (category (catch 'found
+                     (let* ((length (length categories)))
+                       (dotimes (i length)
+                         (let* ((item (elt categories i))
+                                (id (map-elt item 'id))
+                                (name (map-elt item 'name)))
+                           (when (eq id category-id)
+                             (throw 'found name))))))))
+    category))
+
+(defun elfeed-protocol-owncloud--parse-feeds (host-url content)
+  "Parse the feeds JSON buffer and fill results to db.
+HOST-URL is the host name of ownCloud server.  CONTENT is the result JSON
+content by http request.  Ensure the point in the right place that `json-read'
+could execute.  Return `elfeed-protocol-owncloud-feeds'."
+  (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
+         (feeds (map-elt content 'feeds)))
     (puthash proto-id feeds elfeed-protocol-owncloud-feeds)
     (cl-loop for feed across feeds do
              (let* ((feed-url (map-elt feed 'url))
@@ -127,13 +187,19 @@ right place that `json-read' could execute.  Return
     (elfeed-log 'debug "elfeed-protocol-owncloud: found %s feeds" (length feeds))
     elfeed-protocol-owncloud-feeds))
 
-(defun elfeed-protocol-owncloud--update-feed-list (host-url)
+(defun elfeed-protocol-owncloud--update-feed-list (host-url &optional callback)
   "Update ownCloud News feed list.
-HOST-URL is the host name of ownCloud server."
+HOST-URL is the host name of ownCloud server.  Will call CALLBACK at end."
   (elfeed-log 'debug "elfeed-protocol-owncloud: update feed list")
-  (elfeed-protocol-owncloud-with-fetch
-    (concat host-url elfeed-protocol-owncloud-api-feeds)
-    nil (elfeed-protocol-owncloud--parse-feeds host-url)))
+  (let* ((url (concat host-url elfeed-protocol-owncloud-api-feeds))
+         (parse-feeds-func (lambda ()
+                             (elfeed-protocol-owncloud-with-fetch
+                               url nil
+                               (elfeed-protocol-owncloud--parse-feeds host-url result)
+                               (when callback (funcall callback))))))
+    (if elfeed-protocol-owncloud-fetch-category-as-tag
+        (elfeed-protocol-owncloud--update-categories-list host-url parse-feeds-func)
+      (funcall parse-feeds-func))))
 
 (defun elfeed-protocol-owncloud--get-subfeed-url (host-url feed-id)
   "Get sub feed url for the ownCloud protocol feed HOST-URL and FEED-ID."
@@ -168,23 +234,35 @@ HOST-URL is the host name of ownCloud server."
       (elfeed-log 'error "elfeed-protocol-owncloud: no subfeed for feed url %s" feed-url))
     id))
 
+(defun elfeed-protocol-owncloud--get-subfeed-category-id (host-url feed-id)
+  "Get sub feed category id for the owncloud protocol feed HOST-URL and FEED-ID."
+  (let* ((id (catch 'found
+               (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
+                      (feeds (gethash proto-id elfeed-protocol-owncloud-feeds))
+                      (length (length feeds)))
+                 (dotimes (i length)
+                   (let* ((feed (elt feeds i))
+                          (id (map-elt feed 'id))
+                          (folder-id (map-elt feed 'folderId)))
+                     (when (eq id feed-id)
+                       (throw 'found folder-id))))))))
+    (unless id
+      (elfeed-log 'error "elfeed-protocol-owncloud: no subfeed for feed id %s" feed-id))
+    id))
+
 (defun elfeed-protocol-owncloud-entry-p (entry)
   "Check if specific ENTRY is fetched from ownCloud News."
   (let* ((proto-id (elfeed-protocol-entry-protocol-id entry))
          (proto-type (when proto-id (elfeed-protocol-type proto-id))))
     (string= proto-type "owncloud")))
 
-(defun elfeed-protocol-owncloud--parse-entries (host-url &optional mark-state callback)
+(defun elfeed-protocol-owncloud--parse-entries (host-url content &optional mark-state callback)
   "Parse the entries JSON buffer and fill results to elfeed db.
-Ensure the point in the right place that `json-read' could execute.
-HOST-URL is the host name of ownCloud server.  If MARK-STATE is nil,
-then just not update the :last-modifed, :first-entry-id
-and :last-entry-id values.  If CALLBACK is not nil, will call it with
-the result entries as argument.  Return parsed entries.
-
-User could download items.json from ownCloud manually, for example
-http://myhost.com/items?type=3&batchSize=-1, and import the entries by calling
-`elfeed-protocol-owncloud--parse-entries' in the buffer."
+Ensure the point in the right place that `json-read' could execute.  HOST-URL is
+the host name of ownCloud server.  CONTENT is the result JSON content by http
+request.  If MARK-STATE is nil, then just not update the :last-modifed,
+:first-entry-id and :last-entry-id values.  If CALLBACK is not nil, will call it
+with the result entries as argument.  Return parsed entries."
   (if (> (hash-table-count elfeed-protocol-owncloud-feeds) 0)
       (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
              (unread-num 0)
@@ -198,7 +276,7 @@ http://myhost.com/items?type=3&batchSize=-1, and import the entries by calling
                     (elfeed-protocol-get-first-entry-id proto-id)
                     (elfeed-protocol-get-last-entry-id proto-id)
                     (elfeed-protocol-get-last-modified proto-id))
-        (setq items (map-elt (json-read) 'items))
+        (setq items (map-elt content 'items))
         (setq entries
               (cl-loop for item across items collect
                        (pcase-let* (((map id ('guidHash guid-hash) ('feedId feed-id) ('url entry-url) title
@@ -218,6 +296,10 @@ http://myhost.com/items?type=3&batchSize=-1, and import the entries by calling
                                     (original (elfeed-db-get-entry full-id))
                                     (original-date (and original
                                                         (elfeed-entry-date original)))
+                                    (category-name (when elfeed-protocol-owncloud-fetch-category-as-tag
+                                                     (elfeed-protocol-owncloud--get-category-name
+                                                      host-url
+                                                      (elfeed-protocol-owncloud--get-subfeed-category-id host-url feed-id))))
                                     (autotags (elfeed-protocol-feed-autotags proto-id feed-url))
                                     (fixtags (elfeed-normalize-tags
                                               autotags elfeed-initial-tags))
@@ -226,6 +308,8 @@ http://myhost.com/items?type=3&batchSize=-1, and import the entries by calling
                                               (setq fixtags (delete 'unread fixtags)))
                                             (when starred
                                               (push elfeed-protocol-owncloud-star-tag fixtags))
+                                            (when category-name
+                                              (push (intern category-name) fixtags))
                                             fixtags))
                                     (enclosures (when enclosure-link
                                                   (list (list enclosure-link
@@ -336,14 +420,14 @@ nil, will call it with the result entries as argument."
                             (format elfeed-protocol-owncloud-api-update-subfeed arg
                                     elfeed-protocol-owncloud-maxsize)))))
     (elfeed-protocol-owncloud-with-fetch url-opt nil
-      (elfeed-protocol-owncloud--parse-entries host-url mark-state callback)
+      (elfeed-protocol-owncloud--parse-entries host-url result mark-state callback)
       (run-hook-with-args 'elfeed-update-hooks host-url))
     (when (eq action 'init)
       ;; initial sync, fetch starred entries
       (elfeed-protocol-owncloud-with-fetch url-init-starred nil
         ;; do not remember the last-modifed for starred entries, for
         ;; they always not the last entries.
-        (elfeed-protocol-owncloud--parse-entries host-url nil callback)
+        (elfeed-protocol-owncloud--parse-entries host-url result nil callback)
         (run-hook-with-args 'elfeed-update-hooks url-init-starred)))))
 
 (defun elfeed-protocol-owncloud-reinit (host-url)
@@ -353,10 +437,9 @@ HOST-URL is the host name of ownCloud server.  This may take a long
 time, ensure `elfeed-curl-timeout' is big enough."
   (interactive (list (elfeed-protocol-url
                       (completing-read "Protocol Feed: " (elfeed-protocol-feed-list)))))
-  (elfeed-protocol-owncloud-with-fetch
-    (concat host-url elfeed-protocol-owncloud-api-feeds) nil
-    (elfeed-protocol-owncloud--parse-feeds host-url)
-    (elfeed-protocol-owncloud--do-update host-url 'init)))
+  (elfeed-protocol-owncloud-fetch-prepare
+   host-url
+   (elfeed-protocol-owncloud--do-update host-url 'init)))
 
 (defun elfeed-protocol-owncloud-update-since-timestamp (host-url &optional timestamp)
   "Update entries since special timestamp.
@@ -367,10 +450,9 @@ point to 1 hours ago."
                       (completing-read "Protocol Feed: " (elfeed-protocol-feed-list)))))
   (unless timestamp
     (setq timestamp (- (time-to-seconds) (* 1 3600))))
-  (elfeed-protocol-owncloud-with-fetch
-    (concat host-url elfeed-protocol-owncloud-api-feeds) nil
-    (elfeed-protocol-owncloud--parse-feeds host-url)
-    (elfeed-protocol-owncloud--do-update host-url 'update-since-time timestamp)))
+  (elfeed-protocol-owncloud-fetch-prepare
+   host-url
+   (elfeed-protocol-owncloud--do-update host-url 'update-since-time timestamp)))
 
 (defun elfeed-protocol-owncloud-update-since-id (host-url &optional id)
   "Fetch entries after special id.
@@ -378,10 +460,9 @@ HOST-URL is the host name of ownCloud server.  If ID not provide, will
 update since the last entry id."
   (interactive (list (elfeed-protocol-url
                       (completing-read "Protocol Feed: " (elfeed-protocol-feed-list)))))
-  (elfeed-protocol-owncloud-with-fetch
-    (concat host-url elfeed-protocol-owncloud-api-feeds) nil
-    (elfeed-protocol-owncloud--parse-feeds host-url)
-    (elfeed-protocol-owncloud--do-update host-url 'update-since-id id)))
+  (elfeed-protocol-owncloud-fetch-prepare
+   host-url
+   (elfeed-protocol-owncloud--do-update host-url 'update-since-id id)))
 
 (defun elfeed-protocol-owncloud-update-older (host-url)
   "Fetch older entries which entry id less than :first-entry-id.
@@ -575,14 +656,13 @@ result entries as argument"
       (let* ((proto-id (elfeed-protocol-owncloud-id host-url))
              (last-modified (elfeed-protocol-get-last-modified proto-id))
              (last-entry-id (elfeed-protocol-get-last-entry-id proto-id)))
-        (elfeed-protocol-owncloud-with-fetch
-          (concat host-url elfeed-protocol-owncloud-api-feeds) nil
-          (elfeed-protocol-owncloud--parse-feeds host-url)
-          (if (> last-modified 0)
-              (if elfeed-protocol-owncloud-update-with-modified-time
-                  (elfeed-protocol-owncloud--do-update host-url 'update-since-time last-modified callback)
-                (elfeed-protocol-owncloud--do-update host-url 'update-since-id last-entry-id callback))
-            (elfeed-protocol-owncloud--do-update host-url 'init nil callback)))))))
+        (elfeed-protocol-owncloud-fetch-prepare
+         host-url
+         (if (> last-modified 0)
+             (if elfeed-protocol-owncloud-update-with-modified-time
+                 (elfeed-protocol-owncloud--do-update host-url 'update-since-time last-modified callback)
+               (elfeed-protocol-owncloud--do-update host-url 'update-since-id last-entry-id callback))
+           (elfeed-protocol-owncloud--do-update host-url 'init nil callback)))))))
 
 (provide 'elfeed-protocol-owncloud)
 
