@@ -45,12 +45,12 @@ then the published state in Tiny Tiny RSS will be synced, too."
   :group 'elfeed-protocol
   :type 'boolean)
 
-(defvar elfeed-protocol-ttrss-feeds (make-hash-table :test 'equal)
-  "Feed list from Tiny Tiny RSS, will be filled before updating operation.")
-
 (defvar elfeed-protocol-ttrss-categories (make-hash-table :test 'equal)
   "Category list from Tiny Tiny RSS, will be used to tag entries with their TTRSS category.")
 
+(defvar elfeed-protocol-ttrss-feeds (make-hash-table :test 'equal)
+  "Feed list from Tiny Tiny RSS, will be filled before updating operation.")
+
 (defconst elfeed-protocol-ttrss-api-base "/api/")
 (defconst elfeed-protocol-ttrss-api-max-limit 200)
 (defconst elfeed-protocol-ttrss-api-status-ok 0)
@@ -217,14 +217,13 @@ result JSON content by http request.  Return cached
     elfeed-protocol-ttrss-categories))
 
 (defun elfeed-protocol-ttrss--get-category-name (host-url category-id)
-  "Return category name from HOST-URL (symbol) for CATEGORY-ID."
-  (elfeed-log 'debug "elfeed-protocol-ttrss: Fetching category name for id %s" category-id)
+  "Return category name from HOST-URL for CATEGORY-ID."
   (let* ((proto-id (elfeed-protocol-ttrss-id host-url))
          (categories (gethash proto-id elfeed-protocol-ttrss-categories))
          (category (cl-some (lambda (category)
                               (let ((id (cdr (assoc 'id category)))
                                     (category-name (cdr (assoc 'title category))))
-                                (if (eq id category-id) (intern category-name)))) categories)))
+                                (if (eq id category-id) category-name))) categories)))
     category))
 
 (defun elfeed-protocol-ttrss--update-feed-list (host-url &optional callback)
@@ -236,14 +235,14 @@ at end."
                       ("sid" . ,elfeed-protocol-ttrss-sid)
                       ("cat_id" . "-3")))
          (data (json-encode-alist data-list))
-         (parse-feeds (lambda ()
-                        (elfeed-protocol-ttrss-with-fetch
-                          host-url "POST" data
-                          (elfeed-protocol-ttrss--parse-feeds host-url content)
-                          (when callback (funcall callback))))))
+         (parse-feeds-func (lambda ()
+                             (elfeed-protocol-ttrss-with-fetch
+                               host-url "POST" data
+                               (elfeed-protocol-ttrss--parse-feeds host-url content)
+                               (when callback (funcall callback))))))
     (if elfeed-protocol-ttrss-fetch-category-as-tag
-        (elfeed-protocol-ttrss--update-categories-list host-url parse-feeds)
-      (funcall parse-feeds))))
+        (elfeed-protocol-ttrss--update-categories-list host-url parse-feeds-func)
+      (funcall parse-feeds-func))))
 
 (defun elfeed-protocol-ttrss--parse-feeds (host-url content)
   "Parse the feeds JSON buffer and fill results to db.
@@ -282,7 +281,7 @@ result JSON content by http request.  Return
     url))
 
 (defun elfeed-protocol-ttrss--get-subfeed-id (host-url feed-url)
-  "Get sub feed id the ttrss protocol feed HOST-URL and FEED-URL."
+  "Get sub feed id for the ttrss protocol feed HOST-URL and FEED-URL."
   (let* ((id (catch 'found
                (let* ((proto-id (elfeed-protocol-ttrss-id host-url))
                       (feeds (gethash proto-id elfeed-protocol-ttrss-feeds))
@@ -298,7 +297,7 @@ result JSON content by http request.  Return
     id))
 
 (defun elfeed-protocol-ttrss--get-subfeed-category-id (host-url feed-id)
-  "Get sub feed category id the ttrss protocol feed HOST-URL and FEED-ID."
+  "Get sub feed category id for the ttrss protocol feed HOST-URL and FEED-ID."
   (let* ((category-id
           (catch 'found
             (let* ((proto-id (elfeed-protocol-ttrss-id host-url))
@@ -313,7 +312,7 @@ result JSON content by http request.  Return
     category-id))
 
 (defun elfeed-protocol-ttrss--get-subfeed-id-by-title (host-url feed-title)
-  "Get sub feed id the ttrss protocol feed HOST-URL and FEED-TITLE."
+  "Get sub feed id for the ttrss protocol feed HOST-URL and FEED-TITLE."
   (let* ((id (catch 'found
                (let* ((proto-id (elfeed-protocol-ttrss-id host-url))
                       (feeds (gethash proto-id elfeed-protocol-ttrss-feeds))
@@ -398,8 +397,10 @@ it with the result entries as argument.  Return parsed entries."
                                     (original (elfeed-db-get-entry full-id))
                                     (original-date (and original
                                                         (elfeed-entry-date original)))
-                                    (category-id (elfeed-protocol-ttrss--get-subfeed-category-id host-url feed-id))
-                                    (category-name (elfeed-protocol-ttrss--get-category-name host-url category-id))
+                                    (category-name (when elfeed-protocol-ttrss-fetch-category-as-tag
+                                                     (elfeed-protocol-ttrss--get-category-name
+                                                      host-url
+                                                      (elfeed-protocol-ttrss--get-subfeed-category-id host-url feed-id))))
                                     (autotags (elfeed-protocol-feed-autotags proto-id feed-url))
                                     (fixtags (elfeed-normalize-tags
                                               autotags elfeed-initial-tags))
@@ -409,7 +410,7 @@ it with the result entries as argument.  Return parsed entries."
                                             (when starred
                                               (push elfeed-protocol-ttrss-star-tag fixtags))
                                             (when category-name
-                                              (push category-name fixtags))
+                                              (push (intern category-name) fixtags))
                                             (when published
                                               (push elfeed-protocol-ttrss-publish-tag fixtags))
                                             fixtags))
